package hepsim;


import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;
import java.nio.file.*;



/**
Downloading ProMC files in multiple threads from the HepSim library

@author S.Chekanov (ANL)

**/

public class Get {

private static String toDir="";

	/**
	* Start downloading
        * @param inputURL input HepSim URL
        * @param dest destination
	**/
	public Get(final String inputURL, final File dest) {

		URL website = null;
		try {
			website = new URL(inputURL);
			Files.copy(website.openStream(), dest.toPath(), StandardCopyOption.REPLACE_EXISTING);
		} catch (MalformedURLException e) {
			HepSim.ErrorMessage("No such file: "+inputURL);
		} catch (IOException e) {
			HepSim.ErrorMessage("Error when saving file "+dest);
		}



	}



	/**
	Make chunks
	**/
	private static List<List<String[]>> createBatch(List<String[]> originalList,
	                int chunkSize) {
		List<List<String[]>> listOfChunks = new ArrayList<List<String[]>>();
		for (int i = 0; i < originalList.size() / chunkSize; i++) {
			listOfChunks.add(originalList.subList(i * chunkSize, i * chunkSize
			                                      + chunkSize));
		}
		if (originalList.size() % chunkSize != 0) {
			listOfChunks.add(originalList.subList(originalList.size()
			                                      - originalList.size() % chunkSize, originalList.size()));
		}
		return listOfChunks;
	}




	
	
	// main program

	public static void main(String args[]){


		DecimalFormat formatter = new java.text.DecimalFormat("####.##");

		int Nthreads=2; // number of threads for download
		int xlength = args.length;
		// System.out.println(length);
                int nfiles=-1;

		if (xlength <2) {
			HepSim.ErrorMessage("Usage: It takes 2, 3 or 4 arguments:  URL, OUTPUT directory, Nr of threads (optional), Nr of files (optional). Exit!");
			System.exit(1);
		}


		if (xlength == 3) {
			System.out.println("HepSim: Running using "+args[2].trim()+" threads");
                        try  {
                          Nthreads = Integer.parseInt(args[2]);
                        } catch (NumberFormatException e) {
                           HepSim.ErrorMessage("Error: Cannot parse number of threads. It is not integer number! Exit!");
                           System.exit(1);
                        }

                        if (Nthreads>6)  { HepSim.ErrorMessage("Too many threads given. Set back to 2!"); Nthreads=2; }
		}

               if (xlength == 4) {
                        try  {
                           nfiles = Integer.parseInt(args[3]);
                        } catch (NumberFormatException e) {
                           HepSim.ErrorMessage("Error: Cannot parse number of files to download. It is not integer number! Exit!");
                           System.exit(1);
                        }
                }


		String surl=args[0].trim();
		toDir=args[1].trim();

		if (!HepSim.isValidURL(surl)) {
			HepSim.ErrorMessage("HepSim: First argument is not valid URL. Exit!");
			System.exit(0);
		};



		File newDirectory = new File(toDir);
		if(!newDirectory.exists()) {
			boolean result = newDirectory.mkdir();
			if(result) {
				System.err.println("HepSim: The directory "+toDir+" is created!");
			}
		} else {
			System.out.println(HepSim.ANSI_YELLOW+"Warning: The directory \""+toDir+"\" already exists!"+HepSim.ANSI_RESET);
		}




		System.out.println("\nStart downloading in "+Integer.toString(Nthreads)+" threads");
		System.out.println("From: "+surl);
		System.out.println("To:   "+toDir);
		ArrayList<String[]> data=HepSim.getListArraySizes(surl);

                // trim list
                ArrayList<String[]>   data_new= new ArrayList<String[]>();

                if (nfiles==-1) System.out.println("Nr files:   "+"All "+Integer.toString(data.size())+" wll be downloaded");
                else    System.out.println("Nr files:   "+"Sample with "+Integer.toString(nfiles)+" files will be downloaded");


                // trimp
                for (int j=0; j<data.size(); j++) {
                         if (nfiles>-1)
                                if (j>nfiles-1) break;
                        data_new.add(data.get(j));
                }


		List<List<String[]>> batchfiles=createBatch(data_new, Nthreads);


		//List<String> fileNames = new ArrayList<String>();
		//fileNames.add("http://mc.hep.anl.gov/asc/hepsim/events/pp/14tev/qcd/qcd_herwigpp_pt1tev/mu140/qcd_herwigpp_pt1tev_mu140_001.promc");
		//fileNames.add("http://mc.hep.anl.gov/asc/hepsim/events/pp/14tev/qcd/qcd_herwigpp_pt1tev/mu140/qcd_herwigpp_pt1tev_mu140_002.promc");


		final int bntot=batchfiles.size();

		// loop over bunches in group of  Nthreads;
		for (int n=0; n<bntot; n++) {

			List<String[]> batch=batchfiles.get(n);
			// System.out.println(batch.size());
			List<String> fileNames = new ArrayList<String>();

			int isdone=(int)(  100.0*(n/(double)bntot)  );

			System.out.print(formatter.format(isdone)+"% -> ");
			for (int j=0; j<batch.size(); j++) {
				String[] dd= batch.get(j);
				fileNames.add(dd[0]);
				System.out.print(dd[0]+" ");
			}
			System.out.println("");

			// if files have correct sizes, skip

			long done=0;
			for (int j=0; j<batch.size(); j++) {
				String[] dd= batch.get(j);
				// System.out.println(dd[0]);
				// System.out.println(dd[1]);
				long onlinesize=Long.parseLong(dd[1]);
				File f = new File(toDir+File.separator+dd[0]);
				if (f.exists()){
					long kilobytes = (f.length() / 1024);
					// System.out.println(kilobytes);
					// 4 bytes is a tolerance between java and du
					if (Math.abs(kilobytes-onlinesize)<5) done++;
				}
			}

			// skip this batch since we have done this
		if (done == Nthreads) {System.out.println("Already done: "); continue; };


			List<Thread> threads = new ArrayList<Thread>();
			for (String name : fileNames) {

				final String NAME=name;
				final String inputURL=surl+"/"+name;
				final String destFile=toDir+File.separator+NAME;

				Thread t = new Thread() {
					           @Override public void run() {
						           File dest= new File(destFile);
						           Get d= new Get(inputURL,dest);
					           }

					           // downloadFile(name, toPath); }
				           };
				t.start();
				threads.add(t);


			}
			for (Thread t : threads) {

				try {
					t.join();
				} catch(InterruptedException e) {
					// this is executed when an exception (in this example InterruptedException) occurs
					HepSim.ErrorMessage("Download was interrupted!");
				}

			}


		} // end running in batches



	} // end main




}
